@@ -9,6 +9,8 @@
   #include "po.h"
   #define STOP     error
   #define DTPRINT  Rprintf
+  static char internal_error_buff[256] __attribute__((unused)); // todo: fix imports such that compiler warns correctly #6468
+  #define INTERNAL_STOP(...) do {snprintf(internal_error_buff, 255, __VA_ARGS__); error("%s %s: %s. %s", _("Internal error in"), __func__, internal_error_buff, _("Please report to the data.table issues tracker"));} while (0)
 #endif
 
 typedef void writer_fun_t(const void *, int64_t, char **);
@@ -111,6 +113,7 @@ typedef struct fwriteMainArgs
   int nth;
   bool showProgress;
   bool is_gzip;
+  int gzip_level;
   bool bom;
   const char *yaml;
   bool verbose;
